@@ -209,9 +209,15 @@ void TaskGroup::add(AbstractGroupableItem *item, int insertIndex)
         index = d->members.count();
         if (d->groupManager->separateLaunchers()) {
             if (item->itemType() == LauncherItemType) {
-                // insert launchers together at the head of the list, but still
-                // in the order they appear
-                for (index = 0; index < d->members.count(); ++index) {
+                KUrl lUrl = item->launcherUrl();
+                int urlIdx = d->groupManager->launcherIndex(lUrl);
+
+                int maxIndex = urlIdx;
+                if(maxIndex < 0) {
+                    maxIndex = d->members.count();
+                }
+
+                for (index = 0; index < maxIndex; ++index) {
                     if (d->members.at(index)->itemType() != LauncherItemType) {
                         break;
                     }
